// src/state-store.ts
var StateStore = class {
  elementStore;
  constructor() {
    this.elementStore = /* @__PURE__ */ new Map();
  }
  storeAttributes(elements, attributes) {
    const elems = Array.isArray(elements) ? elements : [elements];
    elems.forEach((el) => {
      const store = this.elementStore.get(el) ?? {};
      attributes.forEach(
        (attr) => store[attr] = el.getAttribute(attr)
      );
      this.elementStore.set(el, store);
    });
  }
  revert() {
    for (const [el, attributes] of this.elementStore.entries()) {
      Object.keys(attributes).forEach((attribute) => {
        const cachedValue = attributes[attribute];
        if (cachedValue === null || cachedValue === "") {
          el.removeAttribute(attribute);
        } else if (cachedValue !== null) {
          el.setAttribute(attribute, cachedValue);
        }
      });
    }
    this.elementStore.clear();
  }
};

// src/responder.ts
var boxSliders = [];
var resizeObserver;
if (typeof ResizeObserver !== "undefined") {
  resizeObserver = new ResizeObserver((sliderElements) => {
    sliderElements.forEach(
      (sliderEl) => boxSliders.find((bs) => bs.el === sliderEl.target)?.reset()
    );
  });
}
var responder = {
  add(boxSlider) {
    boxSliders.push(boxSlider);
    resizeObserver?.observe(boxSlider.el);
  },
  remove(boxSlider) {
    resizeObserver?.unobserve(boxSlider.el);
    boxSliders = boxSliders.filter((b) => b !== boxSlider);
  }
};

// src/transition-queue.ts
var createQueue = () => {
  let next = null;
  function run() {
    const fn = next;
    fn?.().then(() => {
      if (fn === next) {
        next = null;
      } else {
        run();
      }
    });
  }
  return {
    push(fn) {
      const runNow = next === null;
      next = fn;
      if (runNow) {
        run();
      }
    },
    clear() {
      next = null;
    }
  };
};

// src/box-slider.ts
function safeMatchMedia(query) {
  return typeof matchMedia !== "undefined" && matchMedia(query).matches;
}
var defaultOptions = Object.freeze({
  autoScroll: !safeMatchMedia("(prefers-reduced-motion: reduce)"),
  loop: true,
  pauseOnHover: safeMatchMedia("(pointer: fine)"),
  speed: 800,
  startIndex: 0,
  swipe: true,
  swipeTolerance: 30,
  timeout: 5e3
});
var BoxSlider = class {
  _activeIndex;
  _el;
  _effect;
  _stateStore;
  options;
  slides;
  autoScrollTimer;
  eventListeners;
  elListeners;
  isDestroyed;
  transitionQueue;
  /**
   * The currently active slide index
   */
  get activeIndex() {
    return this._activeIndex;
  }
  /**
   * The slider element
   */
  get el() {
    if (this._el === void 0) {
      throw new Error("Slider element is undefined");
    }
    return this._el;
  }
  /**
   * The number of slides in the slider
   */
  get length() {
    return this.slides.length;
  }
  /**
   * Return the value of a option at key
   */
  getOption(key) {
    return this.options[key];
  }
  get stateStore() {
    if (this._stateStore === void 0) {
      throw new Error(
        "State store is undefined, are you trying to interact with a destroyed slider?"
      );
    }
    return this._stateStore;
  }
  get effect() {
    if (this._effect === void 0) {
      throw new Error(
        "Slide effect is undefined, are you trying to interact with a destroyed slider?"
      );
    }
    return this._effect;
  }
  constructor(el, effect, options) {
    this._el = el;
    this._stateStore = new StateStore();
    this.transitionQueue = createQueue();
    this.slides = [];
    this.eventListeners = {};
    this.elListeners = {};
    this.isDestroyed = false;
    this.options = {
      ...defaultOptions,
      ...options
    };
    this._activeIndex = this.options.startIndex;
    this.init(effect);
    this.applyEventListeners();
    responder.add(this);
  }
  /**
   * Reset the slider with optional new settings and effect
   */
  reset(options, effect) {
    this.stopAutoScroll();
    if (typeof this.effect.destroy === "function") {
      this.effect.destroy(this.el, this.slides);
    }
    this.stateStore.revert();
    this.options = {
      ...this.options,
      ...options
    };
    if (options?.startIndex !== void 0 && !Number.isNaN(options.startIndex)) {
      this._activeIndex = options.startIndex;
    }
    this.init(effect || this.effect);
    this.emit("reset");
  }
  /**
   * Show the next slide. If the slider is at the last slide and loop is disabled
   * then the slider will not advance back to the first slide.
   */
  next() {
    const isLastSlide = this.activeIndex === this.slides.length - 1;
    if (isLastSlide && !this.options.loop) {
      return Promise.resolve();
    }
    return this.skipTo(isLastSlide ? 0 : this.activeIndex + 1, false);
  }
  /**
   * Show the previous slide. If the slider is at the first slide and loop is disabled
   * then the slider will not advance back to the last slide.
   */
  prev() {
    const isFirstSlide = this.activeIndex === 0;
    if (isFirstSlide && !this.options.loop) {
      return Promise.resolve();
    }
    return this.skipTo(
      isFirstSlide ? this.slides.length - 1 : this.activeIndex - 1,
      true
    );
  }
  /**
   * Skip to a specific slide index
   */
  skipTo(nextIndex, backwards) {
    return new Promise((resolve) => {
      this.transitionQueue.push(async () => {
        await this.transitionTo(
          nextIndex,
          backwards === void 0 ? nextIndex < this.activeIndex : backwards
        );
        resolve();
      });
    });
  }
  /**
   * Pause the slider from auto scrolling
   */
  pause() {
    this.options.autoScroll = false;
    if (this.autoScrollTimer) {
      this.stopAutoScroll();
      this.emit("pause");
    }
    return this;
  }
  /**
   * Start the slider auto scrolling
   */
  play() {
    this.options.autoScroll = true;
    this.setAutoScroll();
    this.emit("play");
    return this;
  }
  /**
   * Add an event listener for a slider event
   */
  addEventListener(ev, callback) {
    if (!Array.isArray(this.eventListeners[ev])) {
      this.eventListeners[ev] = [];
    }
    this.eventListeners[ev].push(callback);
    return this;
  }
  /**
   * Remove an event listener for a slider event. The listener
   * must be the original function passed to addEventListener
   */
  removeEventListener(ev, callback) {
    const listeners = this.eventListeners[ev]?.filter(
      (cb) => cb !== callback
    );
    this.eventListeners[ev] = listeners;
    return this;
  }
  /**
   * Destroy the slider, remove all event listeners and attributes
   */
  destroy() {
    this.isDestroyed = true;
    this.stopAutoScroll();
    if (this.effect.destroy) {
      this.effect.destroy(this.el, this.slides);
    }
    this.stateStore.revert();
    this.transitionQueue.clear();
    responder.remove(this);
    this.emit("destroy");
    this.eventListeners = {};
    Object.entries(this.elListeners).forEach(
      ([ev, listeners]) => listeners.forEach(
        (listener) => this.el.removeEventListener(ev, listener)
      )
    );
    this.elListeners = {};
    delete this._el;
    delete this._stateStore;
    delete this._effect;
    this.slides.length = 0;
  }
  init(effect) {
    this._effect = effect;
    this.slides = this.getSlides();
    this.stateStore.storeAttributes([this.el, ...this.slides], ["style"]);
    if (this.activeIndex >= this.slides.length) {
      this._activeIndex = 0;
    }
    this.effect.initialize(
      this.el,
      this.slides,
      {
        ...this.options,
        startIndex: this.activeIndex
      },
      this.stateStore
    );
    this.addAriaAttributes();
    if (this.options.autoScroll) {
      this.setAutoScroll();
    }
  }
  addAriaAttributes() {
    this.stateStore.storeAttributes(
      [this.el, ...this.slides],
      ["aria-hidden", "aria-live", "aria-roledescription", "role"]
    );
    this.el.setAttribute("aria-atomic", "false");
    this.el.setAttribute("aria-live", "polite");
    if (!this.el.hasAttribute("role")) {
      this.el.setAttribute("role", "region");
    }
    this.slides.forEach((slide, index) => {
      if (!slide.hasAttribute("role")) {
        slide.setAttribute("role", "group");
      }
      if (!slide.hasAttribute("aria-roledescription")) {
        slide.setAttribute("aria-roledescription", "slide");
      }
      slide.setAttribute(
        "aria-hidden",
        index === this.activeIndex ? "false" : "true"
      );
    });
  }
  getSlides() {
    return [...this.el.children].filter(
      (child) => child instanceof HTMLElement
    );
  }
  async transitionTo(nextIndex, backwards) {
    this.stopAutoScroll();
    if (this.isDestroyed || nextIndex === this.activeIndex || nextIndex < 0 || nextIndex >= this.slides.length) {
      return;
    }
    const settings = {
      el: this.el,
      slides: this.slides,
      speed: this.options.speed,
      currentIndex: this.activeIndex,
      isPrevious: backwards,
      nextIndex
    };
    this._activeIndex = nextIndex;
    this.emit("before", {
      currentIndex: settings.currentIndex,
      nextIndex: settings.nextIndex,
      speed: settings.speed
    });
    try {
      await (this.effect.transition(settings) ?? Promise.resolve());
    } catch {
      return;
    }
    if (this.isDestroyed) {
      return;
    }
    if (this.options.autoScroll) {
      this.setAutoScroll();
    }
    this.slides[settings.currentIndex].setAttribute("aria-hidden", "true");
    this.slides[settings.nextIndex].setAttribute("aria-hidden", "false");
    this.emit("after");
  }
  stopAutoScroll() {
    globalThis.clearTimeout(this.autoScrollTimer);
  }
  emit(ev, payload) {
    const handlers = this.eventListeners[ev];
    if (ev === "destroy") {
      const destroyHandlers = handlers;
      destroyHandlers?.forEach((cb) => cb());
    } else {
      handlers?.forEach(
        (cb) => cb({
          currentIndex: this.activeIndex,
          speed: this.options.speed,
          ...payload
        })
      );
    }
  }
  setAutoScroll() {
    this.stopAutoScroll();
    globalThis.requestAnimationFrame(() => {
      if (!this._el) {
        return;
      }
      this.el.setAttribute("aria-live", "off");
      this.autoScrollTimer = globalThis.setTimeout(
        () => this.next(),
        this.options.timeout
      );
    });
  }
  addElListener(ev, handler) {
    this.el.addEventListener(ev, handler, { passive: true });
    this.elListeners[ev] = this.elListeners[ev] || [];
    this.elListeners[ev].push(handler);
  }
  applyEventListeners() {
    this.addElListener("pointerenter", () => {
      if (this.options.autoScroll && this.options.pauseOnHover) {
        this.pause();
      }
    });
    this.addElListener("pointerleave", () => {
      if (this.options.autoScroll && this.options.pauseOnHover) {
        this.play();
      }
    });
    if (this.options.swipe) {
      this.addSwipeNavigation();
    }
  }
  addSwipeNavigation() {
    let startX = 0;
    this.addElListener("touchstart", (ev) => {
      startX = ev.changedTouches[0].screenX;
    });
    this.addElListener("touchend", (ev) => {
      const distanceX = ev.changedTouches[0].screenX - startX;
      if (Math.abs(distanceX) >= this.options.swipeTolerance) {
        if (distanceX > 0) {
          this.prev();
        } else {
          this.next();
        }
      }
    });
  }
};
var box_slider_default = BoxSlider;
export {
  BoxSlider,
  box_slider_default as default,
  defaultOptions
};
