// src/utils.ts
function applyCss(el, styles) {
  Object.keys(styles).forEach((key) => el.style.setProperty(key, styles[key]));
}

// src/effects/tile/tile-transition.ts
var TILE_CLASS = "bs-tile";
var FRONT_FACE_CLASS = "bs-tile-front";
var BACK_FACE_CLASS = "bs-tile-back";

// src/effects/tile/fade-transition.ts
var FadeTransition = class {
  createTile(tileSettings) {
    const tile = document.createElement("div");
    const front = document.createElement("div");
    const back = document.createElement("div");
    const faces = [front, back];
    tile.classList.add(tileSettings.tileClass);
    applyCss(tile, {
      height: `${tileSettings.height}px`,
      left: `${tileSettings.fromLeft}px`,
      overflow: "hidden",
      position: "absolute",
      top: `${tileSettings.fromTop}px`,
      width: `${tileSettings.width}px`
    });
    faces.forEach((face) => {
      applyCss(face, {
        height: `${tileSettings.boxHeight}px`,
        left: `-${tileSettings.fromLeft}px`,
        position: "absolute",
        top: `-${tileSettings.fromTop}px`,
        width: `${tileSettings.boxWidth}px`
      });
    });
    applyCss(back, { opacity: "0" });
    front.classList.add(tileSettings.frontClass);
    tile.append(front);
    back.classList.add(tileSettings.backClass);
    tile.append(back);
    return tile;
  }
  async transition({ tile, nextFace, delay, duration }) {
    const front = tile.querySelector(`.${FRONT_FACE_CLASS}`);
    const back = tile.querySelector(`.${BACK_FACE_CLASS}`);
    const frontAnimation = front.animate(
      { opacity: nextFace === "front" ? 1 : 0 },
      { duration, fill: "forwards", delay }
    );
    const backAnimation = back.animate(
      { opacity: nextFace === "front" ? 0 : 1 },
      { duration, fill: "forwards", delay }
    );
    await Promise.all([frontAnimation.finished, backAnimation.finished]);
  }
  setTileFace(slide, tileFace) {
    const clone = slide.cloneNode(true);
    clone.style.removeProperty("visibility");
    tileFace.replaceChildren(clone);
  }
};
var fade_transition_default = FadeTransition;

// src/effects/tile/flip-transition.ts
var FlipTransition = class {
  createTile(tileSettings) {
    const tileHolder = document.createElement("div");
    const tile = document.createElement("div");
    const front = document.createElement("div");
    const frontFace = document.createElement("div");
    const back = document.createElement("div");
    const backFace = document.createElement("div");
    const faces = [frontFace, backFace];
    applyCss(tileHolder, {
      height: `${tileSettings.height}px`,
      left: `${tileSettings.fromLeft}px`,
      perspective: "300px",
      position: "absolute",
      top: `${tileSettings.fromTop}px`,
      width: `${tileSettings.width}px`,
      "z-index": `${tileSettings.zIndex}`
    });
    tile.classList.add(tileSettings.tileClass);
    applyCss(tile, {
      height: "100%",
      position: "relative",
      "transform-style": "preserve-3d",
      width: "100%"
    });
    tileHolder.append(tile);
    faces.forEach(
      (face) => applyCss(face, {
        height: `${tileSettings.boxHeight}px`,
        left: `-${tileSettings.fromLeft}px`,
        position: "absolute",
        top: `-${tileSettings.fromTop}px`,
        width: `${tileSettings.boxWidth}px`
      })
    );
    applyCss(front, {
      "backface-visibility": "hidden",
      inset: "0",
      overflow: "hidden",
      position: "absolute",
      "z-index": "1"
    });
    applyCss(back, {
      "backface-visibility": "hidden",
      inset: "0",
      overflow: "hidden",
      position: "absolute",
      transform: "rotateY(180deg)",
      "z-index": "2"
    });
    front.append(frontFace);
    front.classList.add(tileSettings.frontClass);
    tile.append(front);
    back.append(backFace);
    back.classList.add(tileSettings.backClass);
    tile.append(back);
    return tileHolder;
  }
  async transition({ tile, nextFace, delay, duration }) {
    await tile.animate(
      {
        transform: [
          `rotateY(${nextFace === "back" ? 0 : 180}deg)`,
          `rotateY(${nextFace === "back" ? 180 : 0}deg)`
        ]
      },
      {
        delay,
        duration,
        fill: "forwards"
      }
    ).finished;
  }
  setTileFace(slide, tileFace) {
    const clone = slide.cloneNode(true);
    clone.style.removeProperty("visibility");
    tileFace.firstElementChild.replaceChildren(clone);
  }
};
var flip_transition_default = FlipTransition;

// src/effects/tile/tile-slider.ts
var defaultOptions = {
  rowOffset: 50,
  rows: 8,
  tileEffect: "flip"
};
var TileSlider = class {
  options;
  _tileWrapper;
  tileTransition;
  grid;
  activeFace;
  get tileWrapper() {
    if (this._tileWrapper === void 0) {
      throw new Error("TileWrapper is undefined");
    }
    return this._tileWrapper;
  }
  constructor(options) {
    this.options = { ...defaultOptions, ...options };
    this.activeFace = "front";
    this.tileTransition = this.options.tileEffect === "fade" ? new fade_transition_default() : new flip_transition_default();
  }
  initialize(el, slides, options) {
    const slide = slides[options.startIndex || 0];
    const fragment = document.createDocumentFragment();
    this.grid = this.calculateGrid(el);
    if (this._tileWrapper) {
      this.destroy();
    }
    const tileWrapper = document.createElement("div");
    tileWrapper.setAttribute("aria-hidden", "true");
    tileWrapper.dataset.bsElement = "true";
    el.append(tileWrapper);
    this._tileWrapper = tileWrapper;
    if (!"fixed absolute relative".includes(getComputedStyle(el).position)) {
      applyCss(el, { position: "relative" });
    }
    applyCss(this.tileWrapper, {
      position: "absolute",
      inset: "0",
      display: "none"
    });
    const totalTiles = this.grid.rows * this.grid.cols;
    for (let i = 0; i < this.grid.rows; ++i) {
      const fromTop = i * this.grid.tileHeight;
      for (let j = 0; j < this.grid.cols; ++j) {
        const tile = this.tileTransition.createTile({
          backClass: `${TILE_CLASS}-back`,
          boxWidth: this.grid.width,
          boxHeight: this.grid.height,
          fromTop,
          fromLeft: j * this.grid.tileWidth,
          frontClass: `${TILE_CLASS}-front`,
          height: this.grid.tileHeight,
          speed: (options.speed - this.options.rowOffset * (this.grid.rows - 1)) / this.grid.cols,
          tileClass: TILE_CLASS,
          width: this.grid.tileWidth,
          zIndex: totalTiles - (i + j)
        });
        fragment.append(tile);
        this.tileTransition.setTileFace(
          slide,
          tile.querySelector(`.${FRONT_FACE_CLASS}`)
        );
      }
    }
    slides.forEach(
      (s, index) => applyCss(s, {
        position: "absolute",
        visibility: index === options.startIndex ? "visible" : "hidden"
      })
    );
    this.tileWrapper.append(fragment);
  }
  async transition(settings) {
    const tiles = this.tileWrapper.querySelectorAll(`.${TILE_CLASS}`);
    const tileDuration = (settings.speed - this.options.rowOffset * (this.grid.rows - 1)) / this.grid.cols;
    const nextFace = this.activeFace === "front" ? "back" : "front";
    this.tileWrapper.style.setProperty("display", "block");
    settings.slides[settings.currentIndex].style.setProperty(
      "visibility",
      "hidden"
    );
    this.tileWrapper.querySelectorAll(
      `.${nextFace === "front" ? FRONT_FACE_CLASS : BACK_FACE_CLASS}`
    ).forEach(
      (tile) => this.tileTransition.setTileFace(
        settings.slides[settings.nextIndex],
        tile
      )
    );
    for (let i = 0; i < this.grid.rows; i++) {
      for (let j = 0; j < this.grid.cols; j++) {
        const index = i * this.grid.cols + j;
        const tile = tiles[index];
        const transition = this.tileTransition.transition({
          delay: i * this.options.rowOffset + j * tileDuration,
          duration: tileDuration,
          nextFace,
          tile
        });
        if (index === tiles.length - 1) {
          await transition;
          this.activeFace = nextFace;
          this.tileWrapper.style.setProperty("display", "none");
          settings.slides[settings.nextIndex].style.setProperty(
            "visibility",
            "visible"
          );
        }
      }
    }
  }
  destroy() {
    this.tileWrapper.remove();
    delete this._tileWrapper;
  }
  calculateGrid(el) {
    const { width: elWidth, height: elHeight } = getComputedStyle(el);
    const height = Number.parseInt(elHeight, 10) ?? el.offsetHeight;
    const width = Number.parseInt(elWidth, 10) ?? el.offsetWidth;
    const rows = this.options.rows;
    const tileHeight = Math.ceil(height / rows);
    const cols = Math.floor(el.offsetWidth / tileHeight);
    const tileWidth = Math.ceil(width / cols);
    return { cols, height, rows, tileHeight, tileWidth, width };
  }
};
export {
  TileSlider as default,
  defaultOptions
};
